#ifndef CONFIG_H
#define CONFIG_H

#define USB 0
#define LSB 1
#define CW  2

// число диапазонов
#define BAND_COUNT 5

extern const struct _Bands {
  uint8_t   mc;
  long  start, startSSB, end;
  uint8_t sideband;
} Bands[];

#define DEFINED_BANDS \
  {2,  144000000L, 144170000L, 144999999L, USB}, \
  {2,  145000000L, 145100000L, 145999999L, USB}, \
  {70, 143000000L, 143100000L, 143999999L, USB}, \
  {70, 144000000L, 144100000L, 144999999L, USB}, \
  {70, 145000000L, 145100000L, 145999999L, USB}
 
// для режима general coverage
#define FREQ_MIN  144000000L
#define FREQ_MAX  150000000L
#define MENU_DELAY      1000      // задержка вызова меню в сек


// список комманд трансивера. порядок критичен и соответствует обходу физического расположения клавиш слева направо, снизу вверх (7-btn keypad)
enum {
  cmdNone = 0,
  
  // без нажатия Fn  
  cmdBandUp,   // переключение диапазонов или частоты
  cmdBandDown,
  cmdLock,     // Lock freq
  cmdVFOSel,   // VFO A/B
  cmdSplit,    // Split on/off
  cmdRIT,      // RIT
  
  // с нажатой Fn  
  cmdVFOEQ,    // VFO A=B
//  cmdZero,     // устанавливает частоту точно по еденицам кГц. 3623145->3623000
//  cmdDateTime,
  cmdUSBLSB,   // выбор боковой USB/LSB
  cmdAttPre,   // переключает по кругу аттенюатор/увч
  cmdQRP,      // режим уменьшенной выходной мощности
//  cmdHam,      // режим Ham band/General coverage. в режиме Ham кнопки cmdBandUp/Down переключают диапазоны
               // в режиме General coverage - изменяют частоту на +/-1MHz
  cmdMenu,
  cmdZero,     // устанавливает частоту точно по еденицам кГц. 3623145->3623000
  cmdDateTime,
  cmdHam    
};

struct _Settings {
  int id;
  int def_value;
  int min_value;
  int max_value;
  int step;
  char* title;
};

//
#define SETTINGS_COUNT  2

#define SETTINGS_DATA \
  {11,   7,   1,   15,  1, "BRIGHT HIGH"},      /* яркость LCD - 0..15 в обычнойм режиме и powerdown (0 - погашен) */ \
  {12,   1,   0,   15,  1, "BRIGHT LOW"}       /* 0 - постоянно включен */ 

// increase for reset stored to EEPROM settings values to default
#define SETTINGS_VERSION    0x02

// id for fast settings access
enum {
  ID_DISPLAY_BRIGHT_HIGH,
  ID_DISPLAY_BRIGHT_LOW,
};
//



// конфиг "железа"
#include "config_hw.h"

// настройки диапазонов и схемы генерируемых частот 
#include "config_sw.h"

#endif
